package cn.com.qmth.stmms.admin.interceptor;

import cn.com.qmth.stmms.biz.exam.service.SubjectUserService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.interceptor.SessionInterceptor;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Admin模块操作拦截器
 *
 * @author LS
 */
public class AdminInterceptor extends SessionInterceptor {

    @Autowired
    private UserService userService;

    @Autowired
    private SubjectUserService subjectUserService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        return validateAdminUser(buildWebUser(request), request, response);
    }

    private boolean validateAdminUser(WebUser wu, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (wu != null && (wu.getRole() == Role.SYS_ADMIN || wu.getRole() == Role.SCHOOL_ADMIN
                || wu.getRole() == Role.SUBJECT_HEADER || wu.getRole() == Role.SCHOOL_VIEWER)) {
            return true;
        }
        return sessionExpire(request, response, "/login");
    }

    private WebUser buildWebUser(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        Integer userId = session.getWebUserId();
        Role role = session.getWebUserRole();
        if (userId != null && role != null) {
            User user = userService.findById(userId);
            if (user != null && user.getRole() == role && user.getAccessTokenRefreshTime() != null && StringUtils
                    .equals(session.getWebUserToken(), user.getAccessToken()) && !AccessControlUtils
                    .expired(user.getAccessTokenRefreshTime())) {
                WebUser wu = new WebUser(user, null, null);
                if (user.getRole() == Role.SUBJECT_HEADER) {
                    wu.setSubjectCodeSet(subjectUserService.findSubjectCode(user.getId()));
                }
                RequestUtils.setWebUser(request, wu);
                return wu;
            }
        }
        return null;
    }
}
