package cn.com.qmth.stmms.admin.user;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cn.com.qmth.stmms.admin.exam.BaseExamController;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.model.SubjectUser;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamSubjectService;
import cn.com.qmth.stmms.biz.exam.service.SubjectUserService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.biz.user.service.query.UserSearchQuery;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.LogType;
import cn.com.qmth.stmms.common.enums.MarkerExcelError;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.utils.ExportExcel;
import cn.com.qmth.stmms.common.utils.ImportExcel;
import cn.com.qmth.stmms.common.utils.Md5EncryptUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;

@Controller
@RequestMapping("/admin/user/subject")
public class SubjectUserController extends BaseExamController {

    public static final String USER_PASSWORD = "123456";

    @Autowired
    private UserService userService;

    @Autowired
    private SubjectUserService subjectUserService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamService examService;

    @RequestMapping(value = "/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
        try {
            String fileName = "科组长数据导入模板.xlsx";
            List<SubjectUser> list = new ArrayList<SubjectUser>();
            list.add(new SubjectUser());
            new ExportExcel("科组长数据", Marker.class, 2).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            addMessage(redirectAttributes, "导入模板下载失败！失败信息：" + e.getMessage());
        }
        return "redirect:" + "/admin/exam/marker";
    }

    @RequestMapping(value = "/import", method = RequestMethod.POST)
    public String importFile(HttpServletRequest request, MultipartFile file, RedirectAttributes redirectAttributes) {
        WebUser wu = RequestUtils.getWebUser(request);
        try {
            int successNum = 0;
            int failureNum = 0;
            StringBuilder failureMsg = new StringBuilder();
            ImportExcel ei = new ImportExcel(file, 1, 0);
            List<SubjectUser> list = ei.getDataList(SubjectUser.class);

            Map<String, User> userMap = new HashMap<String, User>();
            Map<String, ExamSubject> subjectMap = new HashMap<String, ExamSubject>();
            List<SubjectUser> saveList = new ArrayList<SubjectUser>();
            Integer schoolId = wu.getUser().getSchoolId();
            for (SubjectUser subjectUser : list) {
                if (StringUtils.isBlank(subjectUser.getLoginName())) {
                    failureMsg.append("<br/>科组长 " + subjectUser.getLoginName() + "," + subjectUser.getSubjectCode()
                            + MarkerExcelError.MARKERNAMENULL.getName());
                    failureNum++;
                    continue;
                }
                // ExamSubject examSubject =
                // subjectMap.get(subjectUser.getSubjectCode());
                // if (examSubject == null) {
                // examSubject = subjectService.find(examId,
                // subjectUser.getSubjectCode());
                // if (examSubject == null) {
                // failureMsg.append("<br/>科组长 " + subjectUser.getLoginName() +
                // "," + subjectUser.getSubjectCode()
                // + MarkerExcelError.MARKERNOTCODE.getName());
                // failureNum++;
                // continue;
                // }
                // }
                User user = userMap.get(subjectUser.getLoginName().trim());
                if (user == null) {
                    user = userService.findByLoginName(subjectUser.getLoginName().trim());
                    if (user != null && !schoolId.equals(user.getSchoolId())) {
                        failureMsg.append("<br/>评卷员 " + subjectUser.getLoginName() + "," + subjectUser.getSubjectCode()
                                + MarkerExcelError.MARKERNAMEUSED.getName());
                        failureNum++;
                        continue;
                    }
                    if (user == null) {
                        user = getUser(userMap, subjectUser.getLoginName().trim(), schoolId);
                    }
                }
                // subjectMap.put(examSubject.getCode(), examSubject);
                subjectUser = getSubjectUser(subjectUser.getSubjectCode(), user.getId());
                saveList.add(subjectUser);
            }
            successNum = subjectUserService.batchSave(saveList);
            if (failureNum > 0) {
                failureMsg.insert(0, "，失败 " + failureNum + " 条记录！");
            }
            addMessage(redirectAttributes, "已成功导入 " + subjectMap.size() + " 条科目，绑定 " + successNum + " 条评卷员 "
                    + failureMsg);
        } catch (Exception e) {
            log.error("Batch import marker error!", e);
            addMessage(redirectAttributes, "导入评卷员失败！失败信息：" + e.getMessage());
        }
        return "redirect:" + "/admin/exam-param/marker";
    }

    private SubjectUser getSubjectUser(String subjectCode, Integer userId) {
        SubjectUser subjectUser = subjectUserService.findBySubjectCodeAndUserId(subjectCode, userId);
        if (subjectUser == null) {
            subjectUser = new SubjectUser();
            subjectUser.setSubjectCode(subjectCode);
            subjectUser.setUserId(userId);
            subjectUser.setEnable(true);
        }
        return subjectUser;
    }

    private User getUser(Map<String, User> saveMap, String loginName, Integer schoolId) {
        User user = saveMap.get(loginName);
        if (user == null) {
            user = new User();
            user.setLoginName(loginName);
            user.setName(loginName);
            user.setPassword(Md5EncryptUtils.md5(USER_PASSWORD));
            user.setRole(Role.SUBJECT_HEADER);
            user.setSchoolId(schoolId);
            user.setEnable(true);
            user.setCreatedTime(new Date());
            userService.save(user);
            saveMap.put(loginName, user);
        }
        return user;
    }

    @Logging(menu = "评卷员禁用/启用", type = LogType.UPDATE)
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire(Role.SCHOOL_ADMIN)
    public JSONObject toggle(@RequestParam Integer id, @RequestParam Boolean enable) {
        SubjectUser subjectUser = subjectUserService.findById(id);
        JSONObject obj = new JSONObject();
        if (subjectUser != null) {
            subjectUser.setEnable(enable);
            subjectUserService.save(subjectUser);
            obj.accumulate("success", true);
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "该评卷员不存在");
        }
        return obj;
    }

    @Logging(menu = "科组长绑定", type = LogType.ADD)
    @RequestMapping("/save")
    public String add(Model model, HttpServletRequest request, UserSearchQuery query, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        if (examId > 0) {
            Exam exam = examService.findById(examId);
            query.setSchoolId(exam.getSchoolId());
            query.setRole(Role.SUBJECT_HEADER);
            query = userService.findByQuery(query);
            model.addAttribute("query", query);
            model.addAttribute("subject", subjectService.find(examId, subjectCode));
            return "modules/exam/userSubject";
        } else {
            return "redirect:/admin/exam/list";
        }
    }

    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public String save(@RequestParam String subjectCode, @RequestParam Integer[] userIds, HttpServletRequest request,
            RedirectAttributes redirectAttributes) {
        int examId = getSessionExamId(request);
        if (examId > 0) {
            subjectUserService.save(subjectCode, userIds);
            addMessage(redirectAttributes, subjectCode + "设置成功");
            return "redirect:/admin/user/subject?subjectCode=" + subjectCode;
        } else {
            return "redirect:/admin/exam/list";
        }
    }
}
