package cn.com.qmth.stmms.mark.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.interceptor.SessionInterceptor;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * Mark模块操作拦截器
 *
 * @author XKJ
 */
public class MarkInterceptor extends SessionInterceptor {

    @Autowired
    private UserService userService;

    @Autowired
    private MarkerService markerService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        WebUser wu = RequestUtils.getWebUser(request);
        User user = buildUser(request);
        if (user != null) {
            if (user.getLastLoginTime() == null) {
                // 首次登录，强制重置密码及姓名
                return redirect(request, response, "/mark/reset");
            } else if (wu.getMarkerId() == null) {
                // 通用帐号未选择科目，首先选择强制选择评卷科目
                return redirect(request, response, "/mark/subject-select");
            }
            return true;
        }
        return sessionExpire(request, response, "/login");
    }

    private User buildUser(HttpServletRequest request) {
        WebUser wu = RequestUtils.getWebUser(request);
        StmmsSession session = RequestUtils.getSession(request);
        Integer userId = session.getWebUserId();
        Role role = session.getWebUserRole();
        if (userId != null && Role.MARKER == role) {
            User user = userService.findById(userId);
            if (user != null && user.getAccessTokenRefreshTime() != null
                    && StringUtils.equals(session.getWebUserToken(), user.getAccessToken())
                    && !AccessControlUtils.expired(user.getAccessTokenRefreshTime())) {
                if (wu.getMarkerId() != null && wu.getMarker() == null) {
                    wu.setMarker(markerService.findById(wu.getMarkerId()));
                }
                RequestUtils.setWebUser(request, wu);
                return user;
            }
        }
        return null;
    }
}
